/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
/** Esta clase es la que define un coche virtual, que circular por un mapa de
  * casillas. Para moverse, hay tres mtodos necesarios Girar, Mover y FinMover
  * que deben ser llamados en ese rden desde afuera, para que se puedan resolver
  * todos los problemas de colisiones con flechas y/u otros coches.
  * 
  * Aparte de eso, el constructor recibe varias variables que indican su estado
  * en el mapa (posicin y direccin), y hay un mtodo que debe ser llamado tras
  * el constructor, que autoinicia el coche en el mapa. Esta funcin est separada
  * del constructor, debido a la fora que se generan los coches desde la clase
  * MapaTexto. Es pura comodidad, a pesar de que quedara mejor slo tener una
  * funcin constructora.
  * @see MapaTexto
  * @see Casilla
  */
  
class Coche
{
	private int Pos_x, Pos_y, dx, dy, Num_Coche;
	private boolean he_movido;

	public int get_dir ()
	{
		if (dy < 0) return MapaTexto.DIR_ARRIBA;
		else if (dy > 0) return MapaTexto.DIR_ABAJO;
		else if (dx < 0) return MapaTexto.DIR_IZQUIERDA;
		else return MapaTexto.DIR_DERECHA;
	}

	public void autoinicia (Casilla mapa[][])
	{
		mapa[Pos_x][Pos_y].set_coche (Num_Coche);
	}

	Coche (int x, int y, int dir, int num)
	{
		Num_Coche = num;
		Pos_x = x;
		Pos_y = y;

		switch (dir)
		{
		   case (MapaTexto.DIR_ARRIBA):
				dx = 0;
				dy = -1;
			break;
		   case (MapaTexto.DIR_ABAJO):
				dx = 0;
				dy = 1;
			break;
		   case (MapaTexto.DIR_IZQUIERDA):
				dx = -1;
				dy = 0;
			break;
		   case (MapaTexto.DIR_DERECHA):
				dx = 1;
				dy = 0;
			break;

		}
	}
	public void Girar (Casilla mapa [][], Coche coches [])
	{
	   if (mapa[Pos_x][Pos_y].hay_flecha())
		{
			switch (mapa[Pos_x][Pos_y].get_flecha())
			{
				case (Flecha.FAR):
					dx = 0;
					dy = -1;
				break;

				case (Flecha.FAB):
					dx = 0;
					dy = 1;
				break;

				case (Flecha.FIZ):
					dx = -1;
					dy = 0;
				break;

				case (Flecha.FDE):
					dx = 1;
					dy = 0;
				break;

				case (Flecha.FARD):
					if (dx == -1)
					{
						dx = 0;
						dy = -1;
					}
					else
					{
					 	dx = 1;
						dy = 0;
					}
					break;
					
				case (Flecha.FARI):
					if (dx == 1)
					{
						dx = 0;
						dy = -1;
					}
					else
					{
					 	dx = -1;
						dy = 0;
					}
					break;

				case (Flecha.FABD):
					if (dx ==-1)
					{
						dx = 0;
						dy = 1;
					}
					else
					{
					 	dx = 1;
						dy = 0;
					}
					break;

				case (Flecha.FABI):
					if (dx == 1)
					{
						dx = 0;
						dy = 1;
					}
					else
					{
						dx = -1;
						dy = 0;
					}
					break;
					
				default:
					break;
			}
		}
	}

	public void FinMover ()
	{
		he_movido = false;
	}
	
	public void Mover (Casilla mapa[][], Coche coches[])
	{
		if (he_movido)
			return ;

		/* Ponemos que hemos movido, y borramos la posicin antigua */
		he_movido = true;
		mapa[Pos_x][Pos_y].set_coche (-1);
			
		/* Actualizamos las variables del coche */
		Pos_x += dx;
		Pos_y += dy;
		
		/* nos hemos salido del mapa? */
		if (Pos_x < 0 || Pos_x >= GamePanel.NUM_TILES || Pos_y < 0 || Pos_y >= GamePanel.NUM_TILES)
		{
			// System.out.println ("Coche " + Num_Coche + " saliendo del recinto");
			coches[Num_Coche] = null;
			GamePanel.puntos += 10;
			GamePanel.salvados += 1;
			return ;
		}
	
		/** Controlamos la colisin de los coches, comprobando si la siguiente
		casilla esta ocupada por otro coche y si adems este va en direccin
		contraria para que se produzca la colisin
		*/

		// Si hay coche, y no va de frente, lo forzamos a mover
		if ((mapa[Pos_x][Pos_y].coche >= 0) &&
			(dx * coches[mapa[Pos_x][Pos_y].get_coche()].dx != -1) &&
			(dy * coches[mapa[Pos_x][Pos_y].get_coche()].dy != -1))
		{
			coches[mapa[Pos_x][Pos_y].coche].Mover(mapa, coches);
		}

		// Si a pesar de mover, sigue habiendo coche, lo chocamos.
		if ((mapa[Pos_x][Pos_y].coche >= 0))
		{
			// System.out.println ("Destruyendo el coche " + Num_Coche);
			mapa[Pos_x - dx][Pos_y - dy].explota ();
			GamePanel.destruidos += 1;

			// System.out.println ("Destruyendo el coche " + mapa[Pos_x][Pos_y].get_coche());
			int d =	mapa[Pos_x][Pos_y].get_coche();
			mapa[Pos_x][Pos_y].set_coche (-1);
			mapa[Pos_x][Pos_y].explota ();
			GamePanel.destruidos += 1;

			// Destruccin de los coches.
			coches [d] = null;
			coches[Num_Coche] = null;
			return ;
		}

		/* ponemos la nueva posicin en el mapa */
		mapa[Pos_x][Pos_y].set_coche (Num_Coche);
	}
}

