/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
import java.awt.*;
import java.awt.event.*;

/** Esta clase es el cuadro de dialogo en el que se desarrolla el juego.
	* Contiene el panel especial que alberga los graficos aso como los label
 	* que presentan el tiempo transcurrido, los coches salvados y los puntos
	* conseguidos. Ademas implementa un Thread que comprueba cada segundo si
	* quedan fichas por eliminar.
	* @see java.awt.Dialog
	*/

class GameDialog extends Dialog implements WindowListener, Runnable
{	
	/** Estas tres constantes determinan el paso de nivel,
	  * si el juego ha finalizado por destruir todos los coches, o
	  * si el juego ha terminado por no haber ms niveles
	  */
	public final static int GAME_OVER = 1;
	public final static int FINISHED = 2;
	public final static int N_STAGE = 3;

	/** Este panel existe para alinear correctamente los componentes de la ventana. */
	Panel p;
	/** Estos label son los que mostraran la informacion actualizada del
	  * tiempo transcurrido, la puntuacion conseguida y los coches salvados.
	  */
	Label tiempo;
	Label puntuacion;
	Label saved_coches;
	/** Este es un panel especial que contiene practicamente toda la informacion de la partida.
		* @see java.awt.Panel ImagePanel GamePanel
		*/
	GamePanel paneljuego;	
	/** Este Thread lleva el control del paso de nivel, de la finalizacin
	  * de la partida por la eliminacin de todos los coches o por no haber
	  * ms niveles si quedan o no coches por eliminar.
	  */
	Thread controlador;
	/** Esta variable almacena el Frame "padre" de este Dialog para utilizarlo
		* en la creacion de otro Dialog para el mensaje de victoria
		*/
	Frame Fowner;
	/** Esta variable almacena la frecuencia (en segs./cambio) a la que
		* cambian las fichas
		*/
	int speed;

	/** Variable que almacena el valor del estado del juego determinado por
	  * las constantes GAME_OVER, FINISHED Y N_STAGE
	  */
	public static int estado;

	/** Unico constructor de esta clase, Recibe como parametros el Frame "padre" y la
		* frecuencia de cambio de fichas
		*/
	GameDialog (Frame owner,int _speed)
	{
		super (owner,"CochePun v0.1",true);
		
			Fowner = owner;
			speed = _speed;
			
			addWindowListener (this);
			
			saved_coches = new Label ("Coches salvados: 0");
			tiempo = new Label ("Tiempo -> 0 segs.");
			puntuacion = new Label ("Puntos: "+GamePanel.puntos);
			p = new Panel ();
			paneljuego = new GamePanel (saved_coches,tiempo,puntuacion,speed);
			p.setLayout (new FlowLayout (FlowLayout.LEFT));
			p.add (saved_coches);
			p.add (tiempo);
			p.add (puntuacion);
			setLayout (new BorderLayout (5,5));
					
			add ("South", p);
			add ("Center", paneljuego);
			controlador = new Thread (this);
			controlador.start ();
			pack();
			show();
	}
	/** Sin comentarios */
	public void windowClosing (WindowEvent e)
	{
		dispose ();
	};
	
	/** Este metodo determina si se pasa de nivel si el juego ha terminado
	  * por la eliminacin de todos los coches o por no haber ms niveles,
	  * devolviendo un valor segun sea cada caso.
	  */
	int set_estado ()
	{
	   if(GamePanel.destruidos==GamePanel.N_COCHES)
		   return GAME_OVER;
		else if(GamePanel.salvados>0 && CochePun.nivel == CochePun.MAX_MAPA)
			return FINISHED;
		else
			return N_STAGE;
	}

	/** Este metodo es el que se ejecuta en el Thread de este Dialog (controlador)
		* se ejecuta constantemente comprobando si quedan fichas sin eliminar.
		*/
	public void run ()
	{
		boolean finished = false;

		while (!finished)
		{
			try
			{
				Thread.sleep (500);
			} catch (Exception e) {};

			if (!paneljuego.areMoreCoches() && !finished)
			{
				estado = set_estado();
				finished = true;
				createWinD ();
			}
		}
	}

	/** Este metodo llama al constructor de la ventana de dilogo que
	  * mostrar el mensaje correspondiente en cada momento
	  */
	public void createWinD ()
	{
		paneljuego.controlador.stop();
		new WinDialog (Fowner,paneljuego.getTime());
		dispose ();		
	}
	/** Sin comentarios */	
	public void windowOpened (WindowEvent e){};
	/** Sin comentarios */	
	public void windowActivated (WindowEvent e){};
	/** Sin comentarios */	
	public void windowClosed (WindowEvent e){};
	/** Sin comentarios */	
	public void windowDeiconified (WindowEvent e){};
	/** Sin comentarios */	
	public void windowDeactivated (WindowEvent e){};
	/** Sin comentarios */	
	public void windowIconified (WindowEvent e){};	
}