/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
import java.awt.*;
import java.awt.event.*;
import java.awt.event.InputEvent;

/** Esta clase es la clase principal del juego. Ademas de ser el componente donde
  * se dibujan las fichas contiene toda la informacion necesaria para el desarrolo
  * del juego. Contiene un Thread que controla la actualizacion de tiles, el
  * movimiento de los coches y los metodos para
  * gestionar todos los eventos.
  * @see ImagePanel
  */

class GamePanel extends Panel implements Runnable, MouseListener
{
	/** Esta constante contiene el numero horizontal y vertical de fichas
	  * que se utilizaran.
	  */
   static final int NUM_TILES = 20;

	/** Constante que determina el numero maximo de coches que habr por
	  * cada nivel.
	  */
	static final int N_COCHES = 6;

	boolean primera_vez;
		  
	/** Constante que determina el tamao de cada ficha.*/
	static final int S_TILE = 20;
	
   /** Esta variable es un array de sprites, contiene ordenados en un array bidimensional
	  * todos los sprites que se colocaran ordenados segun
	  * su posicion en el tablero.
	  */
	Casilla mapa[][];

   /** Array de la clase Coche, que almacena las posiciones
	  * x e y en el mapa de los coches, la direccin en la que va
	  * cada uno de ellos y su numero respectivo
	  */
	Coche coches[];

	/** Thread que controla el cambio de las fichas cada x segundos. */
	Thread controlador;
	
	/** Este juego de tiles contiene los graficos de las fichas.
	  * @see TileSet
	  */
	TileSet tiles;

	/** Esta variable va a apuntar al label que va a presentar en pantalla los
		* coches salvados en la partida.
		*/
	Label cars;
	/** Esta variable va a apuntar a la label que va a presentar en pantalla los segundos trancurridos desde el comienzo
		* de la partida.
		*/
	Label tiempo;

	/** Esta variable va a apuntar al label que va a presentar en pantalla los
		* puntos conseguidos en la partida.
		*/
	Label puntuacion;

	/** Esta variable cuenta los segundos trancurridos desde el comienzo de la partida.*/
	int cont = 0;
	
	/** Esta variable almacena los segundos que deben transcurrir enter cambio y cambio de ficha.*/
	int speed;
	
	/** Esta variable cuenta los puntos conseguidos en la partida.*/
	public static int puntos;

	/** Esta variable cuenta los coches salvados en la partida.*/
	public static int salvados;
	
	public static int destruidos;
	/** Unico constructor de la clase, recibe como parametros los label donde
	  * se almacenaran los coches salvados, los segundos transcurridos y la
	  * velocidad de juego en segundos. Este constructor tambien inicializa
	  * el array de sprites y lanza el Thread que controla el juego.
	  */
   public GamePanel (Label _cars,Label _tiempo,Label _puntos,int _speed)
	{
		super ();
		mapa = new Casilla[NUM_TILES][NUM_TILES];
		coches = new Coche[N_COCHES];

		primera_vez = true;
		cars = _cars;
		tiempo = _tiempo;
		puntuacion = _puntos;
		tiles = new TileSet (this);
		speed = _speed;
		
		try {MapaTexto.Lee ("dat/mapa"+ CochePun.nivel +".txt", mapa,
				NUM_TILES, NUM_TILES, coches, N_COCHES);}
		catch (Exception e) {e.printStackTrace(); System.exit (1);}

		controlador = new Thread (this);
		controlador.start ();	
		
		addMouseListener (this);
	}

	/** Este es el metodo que reescribimos de la clase java.awt.Component , todos los componentes
		* lo poseen, este metodo contiene las instrucciones que utilizara este componente para
		*	representarse en pantalla, al reescribirlo cambiamos su aspecto.
		*/

	public void paint (Graphics g)
	{
		fuerza_redibujado();
	/*	 Esto es para forzar el redibujado*/
//		update (g);
	}

	public void fuerza_redibujado ()
	{
		for (int x = 0; x < NUM_TILES; x++)
			for (int y = 0; y < NUM_TILES; y++)
				mapa[x][y].set_dirty (true);

		repaint();
	}

	public void update (Graphics g)
	{
		for (int x = 0; x < NUM_TILES; x++)
			for (int y = 0; y < NUM_TILES; y++)
				mapa[x][y].dibuja_casilla (g, tiles, coches, x*S_TILE, y*S_TILE, this);
	}

	/** Este metodo devuelve true si an quedan coches sin eliminar, de lo contrario
		* devuelve false.
		*/

	public boolean areMoreCoches ()
	{
		int num_coches = 0;

		for (int x = 0; x < N_COCHES; x++)
			if (coches[x] != null)
				num_coches++;

		if (num_coches > 0)
			return true;

		return false;
	}
	
	/** Este metodo devuelve el numero de segundos transcurridos desde el comienzo
		* de la partida.
		*/	
	
	public int getTime ()
	{
		return (cont);		
	}
		
	/** Este es el metodo que creara el Thread. En este metodo se cambian las fichas
		* cada n segundos, y cada segundo se actualiza el tiempo que se lleva de juego.
		*/
		
	public void run ()
	{		
		while (true)
		{
			if (primera_vez)
			{
				fuerza_redibujado();
				try { Thread.sleep (2500); } catch (Exception e){};
				primera_vez = false;
			}
		
			if ((cont % speed)==0)
			{
				for (int x = 0; x < N_COCHES; x++)
					if (coches[x] != null)
						coches[x].Girar (mapa, coches);
						
				for (int x = 0; x < N_COCHES; x++)
					if (coches[x] != null)
						coches[x].Mover (mapa, coches);

				for (int x = 0; x < N_COCHES; x++)
					if (coches[x] != null)
						coches[x].FinMover ();

				repaint ();
			}
			
			try { Thread.sleep (250); } catch (Exception e){};
			cont++;
			cars.setText ("Coches salvados: "+salvados);
			tiempo.setText ("Tiempo -> "+cont+" segs.");
			puntuacion.setText ("Puntos: "+puntos);

		}
	}
	/** Este metodo es tambien un override de un metodo de la clase Component,
		* este metodo devuelve a la JVM el tamao optimo de este componente, para
		* que al hacer un pack() este quede bien dimensionado.
		*/
	public Dimension getPreferredSize ()
	{
		return (new Dimension (S_TILE*NUM_TILES,S_TILE*NUM_TILES));
	}
	
	/** Sin comentarios.
		*/
	public void mouseClicked (MouseEvent e) {}
	/** Sin comentarios.
		*/
	public void mouseExited (MouseEvent e) {};
	/** Sin comentarios.
		*/
	public void mousePressed (MouseEvent e)
	{
		int x = e.getX() / S_TILE;
		int y = e.getY() / S_TILE;

		if ((e.getModifiers() & java.awt.event.InputEvent.BUTTON1_MASK) != 0)
			mapa[x][y].rota_flecha_izquierda();
		else
			mapa[x][y].rota_flecha_derecha();

		repaint();
	}
	/** Sin comentarios.
		*/
	public void mouseReleased (MouseEvent e) {};
	/** Sin comentarios.
		*/
	public void mouseEntered (MouseEvent e) {};
}
